;;  _______________________________________________________________________________________________________________
;;
;;  --------------------   HyperMu'NmGA   ---------------------------------------------------------   HyperMu'NmGA
;;  HyperMu'NmGA   ---------------------------------------------------------   HyperMu'NmGA   ---------------------
;;  _______________________________________________________________________________________________________________



globals     [best_result   ;; the numerical value of the chromosome  closer to the solution
             worst_result  ;; the numerical value of the chromosome more distant from the solution
             eureka-time   ;; the ticks elapsed to find the optimal solution
             eureka!       ;; binary variable turning on when the optimal solution has just reached
             reduction-time;; the ticks required to get a 30% reduction of the mutator genes values average
             donor         ;; the turtle having the best chromosome
             recipient     ;; a random turtle different from the donor
             chromcopy     ;; the copy of the chromosome carried by the donor turtle
             mutcopy       ;; the copy of the mutaror gene carried by the donor turtle
             mate_swcopy   ;; the copy of the mate_switch gene carried by the donor turtle
             mutator_swcopy;; the copy of the mutator_switch gene carried by the donor turtle
             max-mut.av    ;; the highest mutators average value recorded before the problem solution
             min-mut.av    ;; the lowest mutators average value recorded before the problem solution
             a-split       ;; the position of one extreme in a donor's chromosome fragment
             b-split       ;; the position of the second extreme  a in a donor's chromosome fragment
             counter       ;; a counter usefull in different blocks
             ]

turtles-own [chromosome    ;; a string of digits representing a candidate solution to the problem
             mutator       ;; a gene able to impact positively or negatively on the genetic stability of the
                           ;; chromosome and of itself; its strength is quantified by a whole number
             mutator_switch;; a gene able to determine the mutation mode;
             mate_switch   ;; a gene able to determine the reproduction mode;
                           ;; it harbours a binary variable: 0 for asexual reproduction, 1 for sexual reproduction
             fitness       ;; the numerical value of the chromosomal strings (if string-value fitness-function mode)
                           ;; or the Hamming distance complement (i.e. the proximity) to the best chromosomal
                           ;; string (if hamming fitness-function mode)
             ]



;;  ----------   SETUP PROCEDURES   -------------------------------------------------------------------------------
;;  ---------------------------------------------------------------------------------------------------------------

to setup                  ;; reset parameters and create a number of turtles set by the user
  clear-all
  set counter 0
  reset-ticks
  create-turtles turtles-number [
    set shape "circle 2"
    set size 2.3
    fd random 15
    genotype/fenotype-construction
   ]
  message0
  set eureka-time 0
end

to genotype/fenotype-construction
  ;; the genotype (chromosome) and the corresponding phenotype (fitness) of the turtle are initially equal to the
  ;; first answer that is usually given by children to the problem: "what is the greatest number of n digits?"
  set fitness 10 ^ (genes-number - 1)
  ;; the initial formal fitness value is converted into a string: it will be the chromosome structure
  set chromosome word fitness ""
  set label chromosome    ;; chromosome is displayed as label

  ;; in the "hamming" mode, the fitness function is replaced by the complement of the Hamming distance
  ;; (i.e. the "Hamming proximity") to the optimal solution
  if fitness-function = "hamming" [compute-fitness]

  set mutator 1
  set max-mut.av 1
  set min-mut.av 1

  ;; defining mate_switch gene
  if reproduction = "asexual" [set mate_switch 0]
  if reproduction = "sexual" [set mate_switch 1]
  if reproduction = "sex/asex" [set mate_switch random 2]

  ;; defining mutator_switch gene
  if mutate_by = "simple mutagenesis" [set mutator_switch -1]
  if mutate_by = "additive mutator" [set mutator_switch 0]
  if mutate_by = "multiplicative mutator" [set mutator_switch 1]
  if mutate_by = "both mutators" [set mutator_switch random 2]
end


;;  ----------   RUNTIME PROCEDURES   -----------------------------------------------------------------------------
;;  ---------------------------------------------------------------------------------------------------------------

to search
  ask turtles [move]
  ;; thew worse and the best fitness value are detected
  set worst_result min [fitness] of turtles
  set best_result max [fitness] of turtles

  ;; a message is sent to output when the optimal solution is reached  (the solutions are differently detected
  ;; depending on the chosen fitness function)
  if eureka-time = 0 and (best_result = 10 ^ genes-number - 1 or best_result = genes-number) [
      set eureka-time ticks
      set eureka! 1
      message1
    if mutate_by = "simple mutagenesis" [stop]
    ]
  ;; max-mut.av and min-mut.av are recorded before the optimal solution is reached
  if eureka-time = 0  [
    if max-mut.av < mean [mutator] of turtles [set max-mut.av  mean [mutator] of turtles]
    if min-mut.av > mean [mutator] of turtles [set min-mut.av  mean [mutator] of turtles]
    ]
  ;; other STOP conditions
  if eureka-time > 0 [set reduction-time ticks - eureka-time]
  if eureka-time > 0 and mean [mutator] of turtles < 0.7 [message2 stop]
  if eureka-time = 0 and ticks >= 750000 and max [mutator] of turtles < 1 [message3 stop]
  if eureka-time > 0 and min [mutator] of turtles >= 1.5 and reduction-time > 500000  [message4 stop]

  ;; two alternative kinds of selective reproduction are admitted: sexual and asexual
  ;; they take place if the diversity between all chromosomes is not null
  clear-links
  if best_result != worst_result [
    set donor [who] of one-of turtles with [fitness = best_result]
    set chromcopy [chromosome] of turtle donor
    set mutcopy [mutator] of turtle donor
    set mate_swcopy [mate_switch] of turtle donor
    set mutator_swcopy [mutator_switch] of turtle donor
    set recipient [who] of one-of turtles with [fitness != best_result]
    ask turtle donor [
      create-link-to turtle recipient
      if mate_switch = 0 [cloning]
      if mate_switch = 1 [recombination]
    ]
  ]
  ;; mutations occur randomly, the frequency is related to the basic mutation-rate (the mutagenicity
  ;; of the environment) and the total gene number of turtles population
  if random-float 1 < basic_mutation-rate * turtles-number * (genes-number + 3)
     [ask one-of turtles [mutation]
  ]
  tick
  set eureka! 0
end

;; TALK TO ME! ---------------------------------------------------------------------------------------------------

to message0
  output-print "*** HyperMu'NmGA ***"
  output-print " "
end

to message1
  output-print " OPTIMAL SOLUTION"
  output-print " reached on "
  output-print word " tick: " eureka-time
end

to message2
  output-print " "

  output-print " Mutators mean went"
  output-print " down 30% under the"
  output-print  " starting value after"
  output-type " ticks: "
  output-type reduction-time
end

to message3
  output-print " "
  output-print " Low evolutive"
  output-print " potential"
end

to message4
  output-print " "
  output-print " No mutator < 1.5"
  output-print " possible endless"
  output-print " hypermutation status"
end

;; GOOD VIBRATIONS -----------------------------------------------------------------------------------------------

to-report  patches-ahead [rad dis] ; reports to turtles a set of patches ahead
  report [patches in-radius rad] of patch-ahead dis
end

to move
   ifelse (any? other turtles-on patches-ahead 1 1)
    [ bk 1 lt random-float 360]
    [fd 0.001]
end


;; Operator 1.  FITNESS FUNCTIONS
;; ----------------------------------------------------------------------------------------------------------------

to compute-fitness
  set fitness 0
  set counter 0
  if fitness-function = "hamming" [hamming-proximity]
  if fitness-function = "string-value" [
    set fitness read-from-string chromosome]
  set label chromosome
end

to hamming-proximity
  if counter = length chromosome [set counter 0 stop]
  if item counter chromosome = "9" [
    set fitness fitness + 1
    ]
  set counter counter + 1
  hamming-proximity
end


;; Operator 2.  REPRODUCTION
;; ----------------------------------------------------------------------------------------------------------------

to cloning
  ask turtle recipient [
    set chromosome chromcopy
    set mutator mutcopy
    set mutator_switch mutator_swcopy
    set mate_switch mate_swcopy
    compute-fitness
  ]
end

to recombination
  ;; homologous recombination occurs between the chromosome of a randomly chosen turtle (recipient) and the
  ;; chromosome of (one of) the most performing one (donor) that offers a code's fragment of its chromosome
  ;; to the first turtle; the two involved turtles will be highlighted by a link

  set counter 0
  set a-split random genes-number
  set b-split random genes-number
  ask turtle recipient [
    hybridization
    compute-fitness]
end

to hybridization
  ;; the two selected chromosomal strings give place to hybridization according to a mechanism
  ;; similar to the crossing-over following to bacterial conjugation; strings are looped,
  ;; as occur usually in bacterial chromosomes or plasmids
  ifelse a-split < b-split [set chromosome
            replace-item (a-split + counter) chromosome (item (a-split + counter) chromcopy)
            set counter (counter + 1)
            if counter < b-split - a-split [hybridization]]
       [if b-split < a-split [set chromosome
              replace-item ((a-split + counter) mod genes-number)
                  chromosome (item ((a-split + counter) mod genes-number) chromcopy)
              set counter (counter + 1)
              if counter < genes-number - a-split + b-split [hybridization]]
    ]
  ;; if required, the three donor's regulative genes are transferred into the recipient turtle
  if reproduction = "sex/asex" [set mate_switch 1]
  if mutate_by != "simple mutagenesis" [
    if random-float 1 < hym-ratio [set mutator mutcopy]
    if mutate_by = "both mutators" [if random-float 1 < 0.5 [
      set mutator_switch mutator_swcopy]
    ]
  ]
end


;; Operator 3: MUTAGENESIS
;; ----------------------------------------------------------------------------------------------------------------

to mutation
  ;; simple mutagenesis doesn't involve mutator genes
  ifelse mutate_by = "simple mutagenesis" [point-mutation compute-fitness
    if reproduction = "sex/asex" [ ;; mutation of the reproduction mode
      if random-float 1 < 0.5
       [set mate_switch ((mate_switch + 1) mod 2)]
    ]
  ]
  ;; hypermutation can produce a mutation of a structural gene as well as a mutation on the mutator gene;
  ;; the last one determines the mutation number along the chromosome: it can be 0, 1 or more (multimutation)
  [if random-float 1 > hym-ratio [multimutation compute-fitness]
   if random-float 1 < hym-ratio [hypermutation]
   ;; when the population is reproductively heterogeneous, also the mate_switch gene can mutate
   if reproduction = "sex/asex" [
     if random-float 1 * mutator * mu-expressivity > hym-ratio [
       set mate_switch ((mate_switch + 1) mod 2)]  ;; mutation of the reproduction mode
    ]
   ;; when the population is mutationally heterogeneous, also the mutator_switch gene can mutate
   if mutate_by = "both mutators" [
    if random-float 1 * mutator * mu-expressivity > hym-ratio [
      set mutator_switch ((mutator_switch + 1) mod 2)]  ;; mutation of the mutator mode
    ]
  ]
end

to point-mutation
  ;; point-mutations hit randomly only one gene
  set chromosome replace-item random genes-number chromosome word random 10 ""
end

to multimutation
  ;; the multimutations imply different point-mutations in a unique mutation event
  ;;  the number of contemporary point-mutations is determined by the variable stored in the mutator-gene
  ifelse mutator < genes-number * 4
    [repeat mutator [point-mutation]]
    ;; if the mutator value is too high (more than four times the number of the chromosome genes) in order to avoid
    ;; overflow events, the number of point-mutations is cut to four times the number of chromosome genes
    [repeat genes-number * 4 [point-mutation]
  ]
end

to hypermutation
  ;; mutator gene can mutate itself  in two alternative ways (additive or multiplicative, decided by
  ;; mutator_switch gene), the self-mutation effect can be modulated also by the mutator-expressivity parameter
  set mutator mutator + (random 3 - 1) * mutator ^ mutator_switch * mu-expressivity
end
@#$#@#$#@
GRAPHICS-WINDOW
190
10
508
329
-1
-1
9.4
1
10
1
1
1
0
1
1
1
-16
16
-16
16
1
1
1
ticks
30.0

BUTTON
0
95
85
128
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
0
10
180
43
turtles-number
turtles-number
0
500
85.0
1
1
NIL
HORIZONTAL

SLIDER
510
10
695
43
genes-number
genes-number
3
15
12.0
1
1
NIL
HORIZONTAL

BUTTON
35
130
137
163
search / stop
search
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
0

PLOT
445
330
885
590
chromosomes fitness
ticks
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"worst chrom." 1.0 0 -13840069 true "" "if ticks > 0 [plot worst_result]"
"best chrom." 1.0 0 -14070903 true "" "if ticks > 0 [plot best_result]"
"eureka!" 1.0 0 -16050907 true "" "ifelse eureka! = 1 [plot best_result + best_result / 10] \n  [plot 0]"

SLIDER
700
10
885
43
basic_mutation-rate
basic_mutation-rate
0
0.0005
5.0E-6
0.0001
1
NIL
HORIZONTAL

SLIDER
700
95
885
128
mu-expressivity
mu-expressivity
0.01
1.6
0.83
0.01
1
NIL
HORIZONTAL

PLOT
0
330
440
590
mutator genes activity
ticks
NIL
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"highest" 1.0 0 -5825686 true "" "if ticks > 0 [plot max [mutator] of turtles]"
"lowest" 1.0 0 -2139308 true "" "if ticks > 0 [plot min [mutator] of turtles]"
"donor" 1.0 0 -7500403 true "" "if ticks > 0 [plot [mutator] of turtle donor]"
"average" 1.0 0 -1184463 true "" "if ticks > 0 [plot mean [mutator] of turtles]"
"eureka!" 1.0 0 -16050907 true "" "ifelse eureka! = 1 [plot max [mutator] of turtles + 0.3] \n  [plot 0]"

CHOOSER
510
45
695
90
reproduction
reproduction
"asexual" "sexual" "sex/asex"
2

CHOOSER
700
45
885
90
mutate_by
mutate_by
"simple mutagenesis" "additive mutator" "multiplicative mutator" "both mutators"
3

MONITOR
0
170
180
215
best fitness
best_result
1
1
11

PLOT
510
135
695
320
% reproduction 
ticks
NIL
0.0
10.0
0.0
100.0
true
true
"" ""
PENS
"asex" 1.0 0 -13791810 true "" "if ticks > 0 [plot count turtles with [mate_switch = 0] * 100 / turtles-number]"
"sex" 1.0 0 -955883 true "" "if ticks > 0 [plot count turtles with [mate_switch = 1] * 100 / turtles-number]"

MONITOR
380
480
435
525
av.M
mean [mutator] of turtles
3
1
11

OUTPUT
0
215
180
320
11

SLIDER
510
95
695
128
hym-ratio
hym-ratio
0.01
0.2
0.07
0.01
1
NIL
HORIZONTAL

MONITOR
380
525
435
570
max.M 
max-mut.av
3
1
11

PLOT
0
595
885
780
mutators distribution
NIL
NIL
-20.0
20.0
0.0
50.0
true
false
"" ""
PENS
"default" 0.2 1 -14070903 true "" "histogram [mutator] of turtles"

PLOT
700
135
885
320
% mutators 
ticks
NIL
0.0
10.0
0.0
100.0
true
true
"" ""
PENS
"ad_mut" 1.0 0 -4079321 true "" "if ticks > 0 [plot count turtles with [mutator_switch = 0] * 100 / turtles-number]"
"mp_mut" 1.0 0 -3844592 true "" "if ticks > 0 [plot count turtles with [mutator_switch = 1] * 100 / turtles-number]"

CHOOSER
20
45
158
90
fitness-function
fitness-function
"string-value" "hamming"
0

BUTTON
85
95
180
128
search
search
NIL
1
T
OBSERVER
NIL
S
NIL
NIL
0

MONITOR
380
435
435
480
min.M
min-mut.av
3
1
11

@#$#@#$#@
# HyperMu'NmGA 

*** Effect of Hypermutation Cycles in a NetLogo Minimal Genetic Algorithm ***


## WHAT IS IT?

We have previously developed a simple genetic algorithm, its aim was to search for the solution to an elementary arithmetic problem (Leuci, 2020). This model is here modified to explore the effect of a mutator gene and the consequent entrance into a hypermutation state. The phenomenon has a particular concern in the case of tumorigenesis and in a more general way, in cells and tissues submitted to chronic environmental or genomic stress (Leuci, 2017). Nevertheless, the instance is still debated. To shed light on these themes we have designed this model assuming the following statements:

- mutator genes come from an impairment process of genes involved in genome stabilization;
- mutator phenotype is a quantitative character and it can assume positive or negative values: when it is negative it protects against mutations, on the contrary, when positive it enhances the probability of mutations; in the first case they can act as some protooncogenes, able to protect against mutagenesis, repairing DNA mismatch; in the second case they can act as some oncogenes that enhance mutagenesis and tumorigenesis; (Sherr, 2004)
- mutator genes can act on themselves leading toward a more severe mutagenic activity or toward a reversion (triggering a hypermutations cycle).

The model was mainly inspired by the studies of Taddei et al. (1997) and Sniegowski et al. (1997); they led their research on asexually reproducing cells (E. coli) engaged in adaptation experiments. This model allows for simulate also the influence of homologous recombination in the control of mutators element diffusion; this second feature is related to sexual reproduction maintenance, which is another debated issue in evolutionary biology (Becks L, Agrawal AF, 2012; Melián CJ et al. 2012; Raynes Y et al. 2011).


## HOW IT WORKS


Every agent (turtle) owns a “chromosome” representing a candidate solution to the assigned problem. The chromosomes are made up of a string of digits, each one representing a "gene"; genes can mutate and, if the population includes sexual turtles, chromosomes can recombine exchanging fragments (sequence of genes) with the chromosome carried by another turtle. 
So the best answer is searched fundamentally in two steps: mutation and selective reproduction of (part of) the fittest chromosome. Mutations happen randomly with a given frequency altered, if any, by the presence of another genetic element that is the mutator gene: it can enhance or suppress the mutation rate on the chromosome.

Reproduction is selective and involves the most performing agents; it can happen in two alternative ways, named "sexual" and "asexual". Sexual reproduction is linked to a recombination event that takes place between a randomly chosen turtle (recipient) and the most performing one (donor) that offers a fragment of its chromosome to the first one; the two turtles taking part in this process will be highlighted by a link. Recombination leads to the formation of a new hybrid chromosome made up of the chromosome of the recipient turtle where a fragment of genes (i.e. digits) included between two randomly chosen loci, is replaced by the corresponding sequence copied by the donor turtle's chromosome. 
In asexual reproduction, the mutation is the unique source of variability and the donor turtle (i.e. one of the best performers) clones entirely its genome into a recipient turtle.

A more detailed description of the model can be found in the ODD-protocol.


## HOW TO USE IT

- SETUP button creates the selected number of turtles set by TURTLES-NUMBER slider. Each of them carries one chromosome constituted by a string long of no more than 20 genes, as chosen by the user, through the GENES-NUMBER slider. Initially, all chromosomes show low suitability.

- SEARCH buttons launch a block of instructions that after the detection of the current best and worst chromosomes, recall the recombination and mutation operators. There are two kinds of search buttons: SEARCH runs one cycle only; SEARCH/STOP runs the program indefinitely; the same button is useful to stop a run.


### Input Tools


Sliders:

- TURTLES-NUMBER: set the number of turtles and consequently the number of chromosomes;

- GENES-NUMBER: set the number of genes along one chromosome; each gene can store a digit (from 0 to 9);

- BASIC_MUTATION-RATES: set the level of environmental mutagenicity acting on a single gene;

- HyM-RATIO could be seen as the genome fraction occupied by  regulative elements (mutator, mate_switch, mutator_switch)

- Mu-EXPRESSIVITY represents the strength of mutators' genes.

Choosers:

- FITNESS-FUNCTION: in the "string-value" mode, it computes the numerical value of the chromosomal strings; in the "hamming" mode it computes the Hamming proximity to the optimal chromosomal string. The fitness range is different as a consequence of the selected fitness function: it is between 10^(genes-number - 1) and 10^genes-number - 1 for string-value mode; it is between 0 and genes-number for hamming mode.

- REPRODUCE: it can assume three alternative values: sexual, asexual, and sex/asex; the first one leads to a recombination event, and the second one leads to the cloning of the entire genome of one of the best-performing turtles into another turtle. The two modes can coexist in a turtles population ("sex/asex" mode); in a mixed population, the sex gene can mutate but not recombine (in order to maintain balanced the reproductive diversity of the populations).

- MUTATE_BY: it offers four different mutation ways. "simple mutagenesis" is the conventional name for a process producing only one point-mutations on a structural gene; the frequency of simple mutagenesis is constant. If mutagenesis does not follow a "simple" way, the mutations weight can mutate for each turtle: the default value is 1: in this case, the mutator allele produces just point mutations; when it will be lower than 1 the mutator allele behaves as an "antimutator"; when it is higher than 1 he mutator allele behaves as a "multimutator" producing more point mutations (as many as the whole part of its numerical value); mutations can beat mutator itself, leading toward hypermutation.  In this case, the chooser offers three options "additive mutator" makes the mutator gene act on itself as an additive factor that randomly increases or decreases itself operating as an algebraic addend; on the contrary, "multiplicative mutator" acts on itself as an algebraic coefficient. The third mode allows the coexistence of both mutational modes in the turtles' population.


### Output Tools

Monitors:
 
- BEST FITNESS displays the current best candidate solution (chromosome string);  
- AV.MU displays the current mean of mutator genes value inside the turtles' population;
- MIN.M displays the lowest average of mutators' values recorded during one experiment;
- MAX.M displays the highest average of mutators' values recorded during one experiment;

Plots:

- CHROMOSOMES FITNESS plots the variation of the best and the worse chromosome 
- % REPRODUCTION plots the percentage of asexually and sexually reproducing turtles
- % MUTATORS plots the percentage of additive and multiplicative mutating turtles
- MUTATORS DISTRIBUTION is a histogram showing the distribution of the mutator genes values between 15 and -15.

Output:

- a display shows some relevant notes about the state of one experiment.


## THINGS TO NOTICE

- The genes along the chromosome should be considered as structural genes, because they all codify for a candidate solution to the problem. Regulative genes are not located along the chromosome.

- When HyM-RATIO is null hypermutation cycles are suspended that is mutator genes do not mutate and their activity does not undergo to variation

- During reproduction (sexual, asexual, or both) the turtle that is taken as the recipient one is always a randomly chosen turtle (not necessarily the worst performing).

- The code includes some stop conditions calibrated for experiments led with BASIC_MUTATION-RATES 5E-6, TURTLES-NUMBER 85, GENES-NUMBER 12.
	- In presence of mutator genes, often their frequency and activity can increase, so it is interesting to follow their fate after the best solution is reached; in this case, the experiment is stopped when the average value of the mutator genes is reduced by 30% with respect to the default value (i.e. 1); but if this condition is not reached at 500000 ticks after the problem solution, muatators’ value reduction is considered potentially endless and the experiment is stopped.
	- If the mean of mutators genes value is under one and the ticks number is more than 750000 in the condition we have used in our experiments program the experiment execution is stopped because the evolutive potential is very low and is unlikely an ultimate solution could be found.
	-  If mutator genes are not active, the experiment ends when the best solution is reached.



## THINGS TO TRY

A first step could be to search the relationship between the BASIC_MUTATION-RATE (environmental mutagenicity) and the TICKS' number you need to get the optimal solution under "simple mutagenesis" setting, with or without recombination; the range of BASIC_MUTATION-RATE values could be set between 1E-8 and 1E-4: indeed, according to Peck, K. M. and Lauring A. S. (2018) DNA viruses typically have mutation rates on the order of 1E-8 to 1E-6 substitutions per nucleotide site per cell infection, RNA viruses are genetically more unstable having mutation rates on the order of 1E-6 to 1E-4 substitutions per nucleotide site per cell infection.

A second step could be to check the effectiveness of the different kinds of selective reproduction. It looks like nature favourites sexual reproduction but it is still debated why. A hypothesis is that the role of sexual reproduction and homologous recombination is to pack favourable traits and unfavourable traits into distinct genomes, so that selection can act more rapidly and more precisely (Lensky, 2001). 

During adaptation processes, an important stage could be the emergence of the mutator or antimutator phenotypes and the consequent adjustment of the mutation rate operated by selection; this feature shows a particular interest when occurs under stress (Hoffmann and Hercus, 2000). If it is allowed to replace a biological challenge with a computational one,  the model would offer a set of tools to design a plan of experiments to study the different kinds of emerging dynamics.


## EXTENDING THE MODEL

The model could be extended in order to study the mutator genes' behaviour and the reproduction mode in the background of a more challenging problem-solving process.


## RELATED MODELS

1. Leuci, Cosimo (2017). “Starfish Planet” (Version 3) Info Tab. NetLogo Modeling Commons. 
http://modelingcommons.org/browse/one_model/4949#model_tabs_browse_nlw
2. Leuci, Cosimo (2020, January 30). “MGA - Minimal Genetic Algorithm” (Version 1.1.0). CoMSES Computational Model Library. Retrieved from: https://doi.org/10.25937/db7w-zt41


## CREDITS AND REFERENCES

1. Becks, L., Agrawal, A.F. (2012, May) "The Evolution of Sex Is Favoured During Adaptation to New Environments". PLoS Biol. 10 (5): e1001317. https://doi.org/10.1371/journal.pbio.1001317

2. Hoffmann, A.A., and Hercus, M.J. (2000, March) "Environmental Stress as an Evolutionary Force" BioScience, 5 (3): 217-226. https://doi.org/10.1641/0006-3568(2000)050[0217:ESAAEF]2.3.CO;2

3. Lensky, R.E.(2001, October 19) Come Fly, and Leave the Baggage Behind" Science 294 (5542): 533-534. https://doi.org/10.1126/science.1066241

4. Melián CJ, Alonso D, Allesina S, Condit RS, Etienne RS (2012, March) "Does Sex Speed Up Evolutionary Rate and Increase Biodiversity?" PLoS Comput. Biol. 8(3): e1002414. https://doi.org/10.1371/journal.pcbi.1002414

5. Peck, K. M. and Lauring A. S. (2018, July) "Complexity of viral mutation rates" Journal of Virology vol. 92 (14): e01031-17. https://www.ncbi.nlm.nih.gov/pmc/articles/PMC6026756/pdf/e01031-17.pdf

5. Raynes Y, Gazzara M R, Sniegowski P D  (2011, June 07) "Mutator dynamics in sexual and asexual experimental populations of yeast" BMC Evolutionary Biology 11: 158 http://www.biomedcentral.com/1471-2148/11/158

6. Sherr, C. J. (2004, January 23) "Principles of Tumor Suppression" Cell, 116: 235–246. https://doi.org/10.1016/S0092-8674(03)01075-4

7. Sniegowski, P., Gerrish, P. and Lenski R. (1997, June) "Evolution of high mutation rates in experimental populations of E. coli" Nature 387: 703–705. https://doi.org/10.1038/42701

8. Taddei, F., Radman, M., Maynard-Smith, J., Toupance, B., Gouyon, P. H., and Godelle, B. (1997, June) "Role of mutator alleles in adaptive evolution" Nature 387: 700–702. https://doi.org/10.1038/42696

9. Wilensky, U. (1999). NetLogo. http://ccl.northwestern.edu/netlogo/. Center for Connected Learning and Computer-Based Modeling, Northwestern Institute on Complex Systems, Northwestern University, Evanston, IL.


## COPYRIGHT AND LICENSE

Copyright 2022 Cosimo Leuci.

This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305, USA.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.2.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiments sv_0.15-0.8" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0.8"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments sv_0.05-0.8" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0.8"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments sv_0.05-0.4" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0.4"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments sv_0.15-1.2" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="1.2"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments sv.hm_b.0.0-0.0" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <exitCondition>eureka-time &gt; 0</exitCondition>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
      <value value="&quot;hamming&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;both mutators&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments sv.hm_s.0.0-0.0" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <exitCondition>eureka-time &gt; 0</exitCondition>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
      <value value="&quot;hamming&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;simple mutagenesis&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments hm_0.05-0.4" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;hamming&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0.4"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments hm_0.05-0.8" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;hamming&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.05"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0.8"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments hm_0.15-0.8" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;hamming&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="0.8"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments hm_0.15-1.2" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;hamming&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="1.2"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments sv_0.1-1" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="1"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments sv_0.1-1.0" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;string-value&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="1"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiments hm_0.1-1.0" repetitions="15" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>search</go>
    <timeLimit steps="1500000"/>
    <metric>best_result</metric>
    <metric>eureka-time</metric>
    <metric>max-mut.av</metric>
    <metric>min-mut.av</metric>
    <metric>reduction-time</metric>
    <enumeratedValueSet variable="basic_mutation-rate">
      <value value="5.0E-6"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="fitness-function">
      <value value="&quot;hamming&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="turtles-number">
      <value value="85"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="genes-number">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="reproduction">
      <value value="&quot;sexual&quot;"/>
      <value value="&quot;asexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mutate_by">
      <value value="&quot;additive mutator&quot;"/>
      <value value="&quot;multiplicative mutator&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="hym-ratio">
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="mu-expressivity">
      <value value="1"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
1
@#$#@#$#@
